#!/usr/bin/python

import sys
import xml.dom.minidom
import os
import zipfile

if __name__ == '__main__':
    if not len(sys.argv) == 2:
        print >> sys.stderr, "Must specify an xpi"
        exit(1)

    dom_doc = xml.dom.minidom.parseString(zipfile.ZipFile(sys.argv[1]).open('install.rdf').read().strip())

    try:
        attr = dom_doc.getElementsByTagName('RDF:Description')[0].attributes['em:id']
    except IndexError:
        attr = dom_doc.getElementsByTagName('Description')[0].attributes['em:id']

    assert attr.value
    print "%s" % attr.value
    exit(0)



#!/usr/bin/python

from optparse import OptionParser
import os
import os.path
import re
import shutil
import subprocess
import sys
import time
import urllib
import xml.dom.minidom
import json
import tempfile
import select

class DependencyNotFound(Exception):
  def __init__(self, depend):
    super(DependencyNotFound, self).__init__(depend)
    self.path = depend[0]
    self.package = depend[1]

  def __str__(self):
    return 'Dependency not found: %s. Please install package %s' % (self.path, self.package)

class InvalidTagError(Exception):
  def __init__(self, tag):
    super(InvalidTagError, self).__init__(tag)
    self.tag = tag

  def __str__(self):
    return "Tag %s is invalid" % self.tag

def do_exec(args, quiet=True, ignore_error=False, cwd=None):
  if quiet == False:
    print 'Running %s%s' % (' '.join([x for x in args]), '' if cwd == None else (' in %s' % cwd))

  out = ''
  p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
  while p.poll() == None:
    (r, w, e) = select.select([p.stdout], [], [], 5)
    for f in r:
      d = f.read()
      out += d
      if quiet == False:
        print d

  if p.returncode != 0 and ignore_error == False:
    raise Exception("Command '%s' returned exit-status %d:\n%s" % (args[0], p.returncode, p.stderr.read()))

  return (p.returncode, out)

def ensure_cache(repo, cache):
  dest = os.path.join(cache, os.path.basename(repo))
  if os.path.isdir(dest):
    (ret, out) = do_exec(['hg', 'summary'], cwd=dest, quiet=True, ignore_error=True)
    if ret == 0:
      print 'Cache location %s exists, using it' % dest
      do_exec(['hg', 'pull', repo], quiet=False, cwd=dest)
      do_exec(['hg', 'update'], quiet=False, cwd=dest)
      return

  if not os.path.isdir(cache):
    os.makedirs(cache)

  print 'Creating cache location %s' % dest
  do_exec(['hg', 'clone', repo, dest], quiet=False)

def do_checkout(source, dest, tag=None):
  dest = os.path.abspath(dest)
  dest_parent = os.path.dirname(dest)
  if dest_parent != '' and not os.path.isdir(dest_parent):
    os.makedirs(dest_parent)

  do_exec(['hg', 'clone', source, dest], quiet=False)

  try:
    args = ['hg', 'update']
    if tag != None:
      args.append('-r')
      args.append(tag)
    do_exec(args, quiet=False, cwd=dest)
  except:
    if tag != None:
      raise Exception("Revision %s not found in %s" % (tag, source))
    raise

def checkout_source(repo, cache, dest, tag=None):
  print '\n'
  print '*** Checking out source from %s%s ***' % (repo, ' using cache from %s' % cache if cache != None else '')
  local_source = None
  if cache != None:
    ensure_cache(repo, cache)
    local_source = os.path.join(cache, os.path.basename(repo))
  source = repo if local_source == None else local_source
  do_checkout(source, dest, tag=tag)
  print '\n'

def get_setting(settings, name, default=None):
  return settings[name] if name in settings else default

class ScopedTmpdir:
  def __enter__(self):
    self._tmpdir = tempfile.mkdtemp()
    print 'Using temporary directory %s' % self._tmpdir
    return self._tmpdir

  def __exit__(self, type, value, traceback):
    print 'Cleaning temporary directory %s' % self._tmpdir
    shutil.rmtree(self._tmpdir)

class ScopedWorkingDirectory:
  def __init__(self, dir):
    self._wd = os.path.abspath(dir)

  def __enter__(self):
    self._saved_wd = os.getcwd()
    if not os.path.isdir(self._wd):
      os.makedirs(self._wd)

    os.chdir(self._wd)

    return self._saved_wd

  def __exit__(self, type, value, traceback):
    os.chdir(self._saved_wd)

class ScopedRename:
  def __init__(self, src, dest):
    self._src = src
    self._dest = dest

  def __enter__(self):
    os.rename(self._src, self._dest)

  def __exit__(self, type, value, traceback):
    os.rename(self._dest, self._src)

class TarballCreator(OptionParser):
  def __init__(self):
    OptionParser.__init__(self, 'usage: %prog [options]')

    self.add_option('-r', '--repo', dest='repo', help='The remote repository from which to pull the main source')
    self.add_option('-c', '--cache', dest='cache', help='A local cache of the remote repositories')
    self.add_option('-l', '--l10n-base-repo', dest='l10nbase', help='The base directory of the remote repositories to pull l10n data from')
    self.add_option('-t', '--tag', dest='tag', help='Release tag to base the checkout on')
    self.add_option('-n', '--name', dest='name', help='The package name')
    self.add_option('-a', '--application', dest='application', help='The application to build')
    self.add_option('-m', '--mozdir', dest='mozdir', help='The location of the Mozilla source', default='')

  def run(self):
    (options, args) = self.parse_args()

    if options.repo == None:
      self.error('Must specify a remote repository')

    if options.name == None:
      self.error('Must specify a package name')

    if options.application == None:
      self.error('Must specify an application')

    if options.cache != None and not os.path.isabs(options.cache):
      options.cache = os.path.join(os.getcwd(), options.cache)

    settings = None
    with open('debian/config/tarball.conf', 'r') as fd:
      settings = json.load(fd)

    DEPENDENCIES = [
      [ 'hg', 'mercurial' ],
      [ 'tar', 'tar' ]
    ]

    print 'Checking dependencies'
    for depend in DEPENDENCIES:
      if os.path.isabs(depend[0]) and not os.access(depend[0], os.X_OK):
        raise DependencyNotFound(depend)
      else:
        found = False
        for path in os.environ['PATH'].split(os.pathsep):
          if os.access(os.path.join(path, depend[0]), os.X_OK):
            found = True
            break
        if found == False:
          raise DependencyNotFound(depend)

    repo = options.repo
    cache = options.cache
    tag = options.tag
    application = options.application
    l10nbase = options.l10nbase
    name = options.name
    mozdir = options.mozdir

    with ScopedTmpdir() as tmpdir:
      with ScopedWorkingDirectory(os.path.join(tmpdir, name)) as saved_wd:

        checkout_source(repo, cache, '', tag=tag)

        need_moz = get_setting(settings, 'need-post-checkout', False)
        if need_moz:
          print '\n'
          print '*** Running checkout script ***'
          moz_local = None
          moz_repo = os.path.join(os.path.dirname(repo), os.path.basename(repo).replace('comm', 'mozilla'))
          if cache != None:
            ensure_cache(moz_repo, cache)
            moz_local = os.path.join(cache, os.path.basename(moz_repo))
          args = [sys.executable, 'client.py', 'checkout']
          if moz_local != None:
            args.append('--mozilla-repo=%s' % moz_local)
          if tag != None:
            args.append('--comm-rev=%s' % tag)
            args.append('--mozilla-rev=%s' % tag)
          do_exec(args, quiet=False)
          print '\n'

        checkout_source('https://hg.mozilla.org/build/compare-locales', cache, os.path.join(mozdir, 'python/compare-locales'), tag=tag)

        # XXX: In the future we may have an additional l10n source from Launchpad
        if l10nbase != None:
          got_locales = {}
          shipped_locales = os.path.join(application, 'locales/shipped-locales')
          all_locales = os.path.join(application, 'locales/all-locales')
          blacklist_file = get_setting(settings, 'l10n-blacklist')

          print '\n\n'
          print '*** Checking out l10n source from %s%s ***' % (l10nbase, ' using cache from %s' % cache if cache != None else '')

          l10ndir = 'l10n'
          if not os.path.isdir(l10ndir):
            os.makedirs(l10ndir)

          with open(os.path.join(l10ndir, 'changesets'), 'w') as changesets:
            for l10nlist in [all_locales, shipped_locales]:
              with open(l10nlist, 'r') as fd:
                for line in fd:
                  locale = line.split(' ')[0].strip()
                  if locale.startswith('#') or locale in got_locales or locale == 'en-US':
                    continue

                  if l10nlist != all_locales:
                    print 'WARNING: Locale %s is not in all-locales. This is an upstream oversight' % locale

                  try:
                    checkout_source(os.path.join(l10nbase, locale), os.path.join(cache, 'l10n') if cache != None else None, 'l10n/' + locale, tag=tag)
                    (ret, out) = do_exec(['hg', 'tip'], cwd='l10n/' + locale, quiet=True)
                    for line in out.split('\n'):
                      if line.startswith('changeset:'):
                        changesets.write('%s %s\n' % (locale, line.split()[1].strip()))
                        print 'Got changeset %s' % line.split()[1].strip()
                        break

                    got_locales[locale] = 1

                  except Exception as e:
                    # checkout_locale will throw if the specified revision isn't found
                    # In this case, omit it from the tarball
                    print >> sys.stderr, 'Failed to checkout %s: %s' % (locale, e)
                    localedir = os.path.join(l10ndir, locale)
                    if os.path.exists(localedir):
                      shutil.rmtree(localedir)

          # When we also use translations from Launchpad, there will be a file
          # containing the additional locales we want to ship (locales.extra??)
          print '\n\n'
          print '*** Checking that required locales are present ***'

          blacklist = {}
          if blacklist_file:
            with open(os.path.join(saved_wd, blacklist_file), 'r') as fd:
              for line in fd:
                locale = line.strip()
                if locale.startswith('#'):
                  continue

                blacklist[locale] = 1

          with open(shipped_locales, 'r') as fd:
            for line in fd:
              line = line.strip()
              if line.startswith('#'):
                continue

              if line == 'en-US':
                print 'Ignoring en-US'
                continue

              locale = line.split(' ')[0].strip()
              platforms = line.split(' ')[1:]

              if blacklist.has_key(locale):
                print 'Ignoring blacklisted locale %s' % locale
                continue

              if len(platforms) > 0:
                for_linux = False
                for platform in platforms:
                  if platform == 'linux':
                    for_linux = True
                    break
                if not for_linux:
                  print 'Ignoring %s (not for linux)' % locale
                  continue

              if not got_locales.has_key(locale):
                raise Exception("Locale %s is missing from the source tarball" % locale)

              print '%s - Yes' % locale

        version = None
        with open(os.path.join(options.application, 'config/version.txt'), 'r') as vf:
          version = re.sub(r'~$', '', re.sub(r'([0-9\.]*)(.*)', r'\1~\2', vf.read().strip()))

        if tag == None:
          (ret, out) = do_exec(['hg', 'tip'], quiet=True)
          for line in out.split('\n'):
            if line.startswith('changeset:'):
              rev = line.split()[1].split(':')[0].strip()
              changeset = line.split()[1].split(':')[1].strip()
              break

          u = urllib.urlopen('%s/pushlog?changeset=%s' % (repo, changeset))
          dom = xml.dom.minidom.parseString(u.read())
          t = time.strptime(dom.getElementsByTagName('updated')[0].firstChild.nodeValue.strip(), '%Y-%m-%dT%H:%M:%SZ')
          version += '~hg%s%s%sr%s' % ('%02d' % t.tm_year, '%02d' % t.tm_mon, '%02d' % t.tm_mday, rev)
          u.close()

          if need_moz:
            # Embed the moz revision in the version number too. Allows us to respin dailies for comm-central
            # even if the only changes landed in mozilla-central
            (ret, out) = do_exec(['hg', 'tip'], cwd='mozilla', quiet=True)
            for line in out.split('\n'):
              if line.startswith('changeset:'):
                version += '.%s' % line.split()[1].split(':')[0].strip()
                break
        else:
          parsed = False
          version_from_upstream = version
          version = ''
          build = None
          for comp in tag.split('_')[1:]:
            if parsed == True:
              raise InvalidTagError(tag)

            if comp.startswith('BUILD'):
              build = re.sub(r'BUILD', '', comp)
              parsed = True
            elif comp.startswith('RELEASE'):
              parsed = True
            else:
              if version != '':
                version += '.'
              version += re.sub(r'~$', '', re.sub(r'([0-9]*)(.*)', r'\1~\2', comp))

            if parsed == True and version == '':
              raise InvalidTagError(tag)

          if version == '':
            raise InvalidTagError(tag)

          if build != None:
            version += '+build%s' % build

          if not version.startswith(version_from_upstream):
            raise InvalidTagError(tag)

        print '\n\n'
        print '*** Upstream version is %s' % version

        print '\n\n'
        print '*** Packing tarball ***'
        with ScopedWorkingDirectory('..'):
          topsrcdir = '%s-%s' % (name, version)
          with ScopedRename(name, topsrcdir):
            args = ['tar', '-jc', '--exclude-vcs']
            for exclude in settings['excludes']:
              args.append('--no-wildcards-match-slash') if exclude['wms'] == False else args.append('--wildcards-match-slash')
              args.append('--exclude')
              args.append(os.path.join(topsrcdir , exclude['path']))
            args.append('-f')
            args.append(os.path.join(saved_wd, '%s_%s.orig.tar.bz2' % (name, version)))
            for include in settings['includes']:
              args.append(os.path.join(topsrcdir, include))

            do_exec(args, quiet=False)

def main():
  creator = TarballCreator()
  creator.run()

if __name__ == '__main__':
  main()



#!/usr/bin/python

import os
import os.path
import sys
from optparse import OptionParser
from subprocess import Popen
import tempfile
import shutil
import atexit
import re
import traceback
import time

skel = {
  '.config/user-dirs.dirs':
'''# This file is written by xdg-user-dirs-update
# If you want to change or add directories, just edit the line you're
# interested in. All local changes will be retained on the next run
# Format is XDG_xxx_DIR="$HOME/yyy", where yyy is a shell-escaped
# homedir-relative path, or XDG_xxx_DIR="/yyy", where /yyy is an
# absolute path. No other format is supported.
# 
XDG_DESKTOP_DIR="$HOME/Desktop"
XDG_DOWNLOAD_DIR="$HOME/Downloads"
XDG_TEMPLATES_DIR="$HOME/Templates"
XDG_PUBLICSHARE_DIR="$HOME/Public"
XDG_DOCUMENTS_DIR="$HOME/Documents"
XDG_MUSIC_DIR="$HOME/Music"
XDG_PICTURES_DIR="$HOME/Pictures"
XDG_VIDEOS_DIR="$HOME/Videos"''',

  '.config/gnome-session/sessions/test.session':
'''[GNOME Session]
Name=Test Session
RequiredComponents=gnome-settings-daemon;metacity;
DesktopName=GNOME''',

  '.config/autostart/run-test.desktop':
'''[Desktop Entry]
Name=Run Testsuite
Exec=sh -c 'sleep 5; $UBUNTU_MOZ_TEST_RUNNER $UBUNTU_MOZ_TEST_ARGS'
Terminal=false
Type=Application
Categories='''
}

def create_tmpdir():
  tmp = tempfile.mkdtemp()

  def clean():
    shutil.rmtree(tmp)

  atexit.register(clean)

  return tmp

class TestRunHelper(OptionParser):

  def __init__(self, runner, get_runner_parser_cb, pass_args=[], paths=[], need_x=False):
    OptionParser.__init__(self)
    self.__tmpdir = None
    self._xredir = None

    self.root = os.path.dirname(__file__)

    args = iter(sys.argv[1:])
    while True:
      try:
        arg = args.next()
      except StopIteration:
        break

      if arg == '--harness-root-dir':
        try:
          self.root = args.next()
        except StopIteration:
          break
      elif arg == '--xre-path':
        try:
          self._xredir = args.next()
        except StopIteration:
          break

    self._orig_root = self.root
    self._pass_args = pass_args
    self._need_x = need_x

    self._pass_args.append('--xre-path')

    self.add_option('--harness-root-dir',
                    dest='root',
                    help='Override the path to the test harness installation')
    if self._need_x:
      self.add_option('--own-session',
                      action='store_true', dest='wantOwnSession', default=False,
                      help='Run the test inside its own X session')

    if not os.path.exists(os.path.join(self.root, runner)):
      for f in os.listdir(self.root):
        if re.match(r'[a-z\-]*-[0-9\.ab]*\.en-US\.linux-\S*\.tests\.zip', f) or f == 'extra.test.zip':
          os.system('unzip -qu %s -d %s' % (os.path.join(self.root, f), self._tmpdir))

      assert os.path.exists(os.path.join(self._tmpdir, runner))
      self.root = self._tmpdir

    runner = os.path.join(self.root, runner)
    sys.path.insert(0, os.path.dirname(runner))

    for path in reversed(paths):
      sys.path.insert(0, os.path.join(self.root, path))

    self._runner_global = {}
    self._runner_global['__file__'] = runner
    saved_argv0 = sys.argv[0]
    saved_cwd = os.getcwd()
    sys.argv[0] = runner
    os.chdir(self.root)
    try:
      execfile(runner, self._runner_global)

      runner_parser = get_runner_parser_cb(self._runner_global)

      for arg in self._pass_args:
        assert runner_parser.has_option(arg)
        self.add_option(runner_parser.get_option(arg))

    finally:
      sys.argv[0] = saved_argv0
      os.chdir(saved_cwd)

  @property
  def _tmpdir(self):
    if self.__tmpdir != None:
      return self.__tmpdir

    self.__tmpdir = create_tmpdir()
    return self.__tmpdir

  @property
  def xredir(self):
    if self._xredir != None:
      return self._xredir

    if self.root != self._orig_root:
      # This allows us to run with the harness root set to objdir/dist (which should contain
      # a testsuite tarball)
      # Note, our install layout mimics this
      libxul = os.path.join(self._orig_root, 'bin', 'libxul.so')
    else:
      # This allows us to run with the harness root set to objdir/dist/test-package-stage
      libxul = os.path.join(self.root, os.pardir, 'bin', 'libxul.so')

    if os.path.exists(libxul):
      self._xredir = os.path.dirname(libxul)
    else:
      libxul = os.path.join(os.path.dirname(__file__), os.pardir, 'libxul.so')
      if os.path.exists(libxul):
        self._xredir = os.path.dirname(libxul)

    return self._xredir

  def run(self, defaults=[], pre_run_cb=None):
    (options, args) = self.parse_args()

    if os.getenv('DESKTOP_AUTOSTART_ID') == None:
      # If we were started by the session manager, use the current homedir
      os.mkdir(os.path.join(self._tmpdir, 'home'))
      for name in skel:
        os.system('mkdir -p %s' % os.path.dirname(os.path.join(self._tmpdir, 'home', name))) 
        with open(os.path.join(self._tmpdir, 'home', name), 'w+') as f:
          print >>f, skel[name]

      os.environ['HOME'] = os.path.join(self._tmpdir, 'home')

    try:
      if self._need_x and (options.wantOwnSession or os.getenv('DISPLAY') == None):
        import subprocess

        i = 0
        while len(sys.argv) > 0 and len(sys.argv) > i:
          arg = sys.argv[i]
          if arg == '--own-session':
            del sys.argv[i]
          elif arg == '--harness-root-dir':
            del sys.argv[i]
            del sys.argv[i]
          else:
            i += 1

        extra_args = ['--harness-root-dir', self.root]
        if '--xre-path' not in sys.argv and self.xredir != None:
          extra_args.extend(['--xre-path', self.xredir])

        os.environ['UBUNTU_MOZ_TEST_ARGS'] = ' '.join(extra_args + sys.argv[1:])
        os.environ['UBUNTU_MOZ_TEST_RUNNER'] = sys.executable + ' ' + sys.argv[0]
        session_args = ['xvfb-run', '-a', '-s', '-screen 0 1280x1024x24 -extension MIT-SCREEN-SAVER', 'dbus-launch', '--exit-with-session', 'gnome-session', '--session', 'test']
        subprocess.call(session_args, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr)

        if not os.path.exists(os.path.join(os.environ['HOME'], '.test_return')):
          return 1
        with open(os.path.join(os.environ['HOME'], '.test_return'), 'r') as f:
          return int(f.read().strip())

      os.environ['MOZ_PLUGIN_PATH'] = os.path.join(self.root, 'bin/plugins')

      sys.argv = []
      argv = sys.argv
      argv.append(self._runner_global['__file__'])

      for arg in self._pass_args:
        opt = self.get_option(arg)
        action = opt.action
        val = getattr(options, opt.dest)
        if val == opt.default and action != 'append':
          continue
        if action == 'store':
          if val != None:
            argv.extend([arg, str(val)])
        elif action == 'store_true':
          if val == True:
            argv.append(arg)
        elif action == 'store_false':
          if val == False:
            argv.append(arg)
        elif action == 'append':
          if val != None:
            for v in val:
              argv.extend([arg, str(v)])
        else:
          raise RuntimeError('Unexpected argument type with action "%s"' % action)

      for arg in defaults:
        if arg not in argv:
          argv.extend([arg, defaults[arg]() if hasattr(defaults[arg], '__call__') else defaults[arg]])

      if '--xre-path' not in sys.argv and self.xredir != None:
        argv.extend(['--xre-path', self.xredir])

      if pre_run_cb != None:
        pre_run_cb(options, args)

      argv.extend(args)
      os.chdir(self.root)
      return self._runner_global['main']()

    except Exception:
      with open(os.path.join(os.getenv('HOME'), '.test_return'), 'w+') as f:
        print >>f, '1'

      traceback.print_exc()

    finally:
      if os.getenv('DESKTOP_AUTOSTART_ID') != None:
        if not os.path.exists(os.path.join(os.getenv('HOME'), '.test_return')):
          with open(os.path.join(os.getenv('HOME'), '.test_return'), 'w+') as f:
            print >>f, '0'
        i = 0
        while i < 5:
          os.system('gnome-session-quit --logout --no-prompt --force')
          i += 1
          time.sleep(10)
